#include "../global.h"

PhoenixHardware::PhoenixHardware(Epp *e, ScreenOutput *disp, AudioOutput *au, Keyboard *kb, Joystick *js) :
                                               Machine(e, disp, au) {

    keyboard=kb;
    joystick=js;

    vsync=0;

    // This is odd.  If I don't clear the memory, it crashes, but also
    // you can reset the CPU and still continue from where you are,
    // but with no score.

    for (int i=0; i<0x10000; i++) {

    	memory[i]=0;
    }
}

void PhoenixHardware::destroyComponents(void) {

    delete cpu;
    delete vid;
}

void PhoenixHardware::emulate() {

    readROMs();
    createComponents();
    vid->setupDisplay();
    vid->createBanks();

    cpu->setRunTime(1000000/60);

    for(;;) {

        joystick->update();

        cpu->execute();

        vid->update();

        vsync=0x80;

        if(epp->checkInterface()) {

            destroyComponents();
            return;
        }
    }
}

void PhoenixHardware::reset(void) {

    // To do a full reset, I need to wipe the memory.  It's easier
    // to just blatt the whole array, then reload the ROMs.

    for (int i=0; i<0x10000; i++) {

    	memory[i]=0;
    }

    readROMs();

    cpu->reset();
}

uint8 PhoenixHardware::readMem(const Component *comp, const uint16 addr) {

    // Z80 memory, incluing mapped input

    uint8 data=0;

    if(addr >=0x7000 && addr <= 0x73ff) {

        if(joystick->jfire2()) data |= 0x80;
        if(joystick->jleft()) data |= 0x40;
        if(joystick->jright()) data |= 0x20;
        if(joystick->jfire1()) data |= 0x10;
        if(keyboard->keyPressed(Keyboard::k_2)) data |= 0x04;
        if(keyboard->keyPressed(Keyboard::k_1)) data |= 0x02;
        if(keyboard->keyPressed(Keyboard::k_3)) data |= 0x01;

        return ~data;
    }
    else if (addr >=0x7800 && addr <= 0x7bff) {

        data=dip1|vsync;
        vsync=0;
        return data;
    }

    return memory[addr];
}

void PhoenixHardware::writeMem(const Component *comp, const uint16 addr, const uint8 data) {


    // Special:

    // Scroll register

    if (addr>=0x5000 && addr<=0x53ff) {

        vid->setVideoReg(data);
    }
    // Scroll register

    if (addr>=0x5800 && addr<=0x5bff) {

        vid->setScrollReg(data);
    }
    // Foreground

    else if (addr>=0x4000 && addr<=0x43ff) {

        if(data != memory[addr]) {

            memory[addr]=data;
            vid->writeVideoRAM(addr, data);
        }
    }
    // Background

    else if (addr>=0x4800 && addr<=0x4bff) {

        if(data != memory[addr]) {

            memory[addr]=data;
            vid->writeVideoRAM(addr, data);
        }
    }

    // RAM:

    else if (addr>=0x4000 && addr<=0x7fff) {

        memory[addr]=data;
    }

}

uint8 PhoenixHardware::in(const Component *comp, const uint16 port) {

    // No IO!
    return 0;
}

void PhoenixHardware::out(const Component *comp, const uint16 port, const uint8 data) {

    // No IO!
}
